@@ -51,7 +51,7 @@ static zword object_address(zword obj)
 		print_string("@Attempt to address illegal object ");
 		print_num(obj);
 		print_string(".  This is normally fatal.");
-		new_line();
+		reset_window();
 		runtime_error (ERR_ILL_OBJ);
 	}
 
