@@ -12,6 +12,11 @@
 #define DUMB_DFROTZ_H
 
 #include "../common/frotz.h"
+
+#ifndef NO_BASENAME
+#include <libgen.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -19,13 +24,20 @@
 #include <ctype.h>
 #include <time.h>
 
+#include <sys/param.h>
+
+#ifndef MAX
 #define MAX(x,y) ((x)>(y)) ? (x) : (y)
+#endif
+#ifndef MIN
 #define MIN(x,y) ((x)<(y)) ? (x) : (y)
+#endif
 
 /* from ../common/setup.h */
 extern f_setup_t f_setup;
 
 extern bool do_more_prompts;
+extern bool quiet_mode;
 
 /* From input.c.  */
 bool is_terminator (zchar);
