@@ -19,7 +19,6 @@
  * Or visit http://www.fsf.org/
  */
 
-#include <libgen.h>
 #include "dfrotz.h"
 #include "dblorb.h"
 
@@ -33,7 +32,7 @@ static void print_version(void);
 An interpreter for all Infocom and other Z-Machine games.\n\
 \n\
 Syntax: dfrotz [options] story-file [blorb file]\n\
-  -a   watch attribute setting    \t -P   alter piracy opcode\n\
+  -a   watch attribute setting    \t -q   quiet mode (no startup messages)\n\
   -A   watch attribute testing    \t -r <option> Set runtime options\n\
   -f <type> type of format codes  \t -R <path> restricted read/write\n\
   -h # screen height              \t -s # random number seed value\n\
@@ -43,8 +42,10 @@ Syntax: dfrotz [options] story-file [blorb file]\n\
   -O   watch object locating      \t -v   show version information\n\
   -L <file> load this save file   \t -w # screen width\n\
   -m   turn off MORE prompts      \t -x   expand abbreviations g/x/z\n\
-  -p   plain ASCII output only    \t -Z # error checking (see below)\n"
+  -p   plain ASCII output only    \t -Z # error checking (see below)\n\
+  -P   alter piracy opcode\n"
 
+  
 #define INFO2 "\
 Error checking: 0 none, 1 first only (default), 2 all, 3 exit after any error.\n\
 For more options and explanations, please read the manual page.\n\n\
@@ -57,6 +58,7 @@ static int user_random_seed = -1;
 static int user_tandy_bit = 0;
 static bool plain_ascii = FALSE;
 
+bool quiet_mode;
 bool do_more_prompts;
 
 /*
@@ -75,9 +77,10 @@ void os_process_arguments(int argc, char *argv[])
 	zoptarg = NULL;
 
 	do_more_prompts = TRUE;
+	quiet_mode = FALSE;
 	/* Parse the options */
 	do {
-		c = zgetopt(argc, argv, "aAf:h:iI:L:moOpPs:r:R:S:tu:vw:xZ:");
+		c = zgetopt(argc, argv, "aAf:h:iI:L:moOpPqr:R:s:S:tu:vw:xZ:");
 		switch(c) {
 		case 'a':
 			f_setup.attribute_assignment = 1;
@@ -133,6 +136,9 @@ void os_process_arguments(int argc, char *argv[])
 		case 'p':
 			plain_ascii = 1;
 			break;
+		case 'q':
+			quiet_mode = 1;
+			break;
 		case 'r':
 			dumb_handle_setting(zoptarg, FALSE, TRUE);
 			break;
@@ -180,45 +186,55 @@ void os_process_arguments(int argc, char *argv[])
 		os_quit(EXIT_SUCCESS);
 	}
 
-	switch (f_setup.format) {
-	case FORMAT_IRC:
-		printf("Using IRC formatting.\n");
-		break;
-	case FORMAT_ANSI:
-		printf("Using ANSI formatting.\n");
-		f_setup.format = FORMAT_ANSI;
-		break;
-	case FORMAT_BBCODE:
-		printf("Using Discourse BBCode formatting.\n");
-		f_setup.format = FORMAT_BBCODE;
-		break;
-	case FORMAT_UNKNOWN:
-		printf("Unknown formatting \"%s\".\n", format_orig);
-		f_setup.format = FORMAT_NORMAL;
-		break;
-	case FORMAT_DISABLED:
-		printf("Format selection disabled at compile time.\n");
-		f_setup.format = FORMAT_NORMAL;
-		break;
-	default:
-		break;
+	if (!quiet_mode) {
+		switch (f_setup.format) {
+		case FORMAT_NORMAL:
+			printf("Using normal formatting.\n");
+			break;
+		case FORMAT_IRC:
+			printf("Using IRC formatting.\n");
+			break;
+		case FORMAT_ANSI:
+			printf("Using ANSI formatting.\n");
+			break;
+		case FORMAT_BBCODE:
+			printf("Using Discourse BBCode formatting.\n");
+			f_setup.format = FORMAT_BBCODE;
+			break;
+		case FORMAT_UNKNOWN:
+			printf("Unknown formatting \"%s\".  Using normal formatting instead.\n", format_orig);
+			break;
+		case FORMAT_DISABLED:
+			printf("Format selection disabled at compile time.\n");
+			break;
+		default:
+			printf("Something else happened with format selection.\n");
+			printf("This should not happen.\n");
+			break;
+		}
 	}
-	if (f_setup.format == FORMAT_NORMAL)
-		printf("Using normal formatting.\n");
+	if (f_setup.format == FORMAT_UNKNOWN || FORMAT_DISABLED)
+		f_setup.format = FORMAT_NORMAL;
 
 	/* Save the story file name */
 	f_setup.story_file = strdup(argv[zoptind]);
-	f_setup.story_name = strdup(basename(argv[zoptind]));
 
+#ifdef NO_BASENAME
+	f_setup.story_name = strdup(f_setup.story_file);
+#else
+	f_setup.story_name = strdup(basename(argv[zoptind]));
+#endif
 	if (argv[zoptind+1] != NULL)
 		f_setup.blorb_file = strdup(argv[zoptind+1]);
 
-	printf("Loading %s.\n", f_setup.story_file);
+	if (!quiet_mode) {
+		printf("Loading %s.\n", f_setup.story_file);
 
 #ifndef NO_BLORB
 	if (f_setup.blorb_file != NULL)
 		printf("Also loading %s.\n", f_setup.blorb_file);
 #endif
+	}
 
 	/* Now strip off the extension */
 	p = strrchr(f_setup.story_name, '.');
