@@ -47,7 +47,8 @@ extern FILE *blorb_fp;
 
 /* various data */
 bool m_tandy = 0;
-int m_v6scale;
+int m_v6scale_x;
+int m_v6scale_y;
 double m_gfxScale_w = 1.0;
 double m_gfxScale_h = 1.0;
 ulong m_defaultFore;
@@ -64,7 +65,6 @@ char *m_fontdir = NULL;
 bool m_aafonts = 0;
 char m_names_format = 0;
 char *m_reslist_file = NULL;
-char *m_setupfile = ".sfrotzrc";
 extern int m_frequency;
 
 bool sdl_active;
@@ -77,13 +77,17 @@ static char *ResDir = "./";
 static char *ResPict = "PIC%d";
 static char *ResSnd = "SND%d";
 
-int AcWidth = 640, AcHeight = 400;
+int AcWidth  = DEFAULT_WIDTH;
+int AcHeight = DEFAULT_HEIGHT;
+
 int option_scrollback_buffer = 0;
+bool option_disable_color = 0;
 bool m_adaptiveMode = FALSE;
 
+bb_resolution_t *reso;
+
 static void checkwidths()
 {
-	bb_resolution_t *reso;
 	reso = bb_get_resolution(blorb_map);
 	if (reso) {
 		/* ignore small resolution hints */
@@ -122,6 +126,7 @@ int sf_load_resources(void)
 		checkwidths();
 		bb_count_resources(blorb_map, bb_ID_Pict, &countedpics, NULL,
 				   &maxlegalpic);
+
 		releaseno = bb_get_release_num(blorb_map);
 		if (bb_load_chunk_by_type (blorb_map,
 					   bb_method_Memory,
@@ -393,7 +398,7 @@ void sf_readsettings(void)
 {
 	char *p;
 
-	sf_InitProfile(m_setupfile);
+	sf_InitProfile(f_setup.config_file);
 
 	m_aafonts = sf_GetProfileInt("Fonts", "antialias", 0);
 	m_fontdir = sf_GetProfileString("Fonts", "fontdir", NULL);
@@ -412,11 +417,7 @@ void sf_readsettings(void)
 	ResPict = sf_GetProfileString("Resources", "Pict", ResPict);
 	ResSnd = sf_GetProfileString("Resources", "Snd", ResSnd);
 
-	if (f_setup.interpreter_number == 0) {
-		z_header.interpreter_number =
-		    sf_GetProfileInt("Interpreter", "Number", INTERP_AMIGA);
-	} else
-		z_header.interpreter_number = f_setup.interpreter_number;
+	z_header.interpreter_number = sf_GetProfileInt("Interpreter", "Number", INTERP_AMIGA);
 
 	f_setup.err_report_mode =
 	    sf_GetProfileInt("Interpreter", "Error Reporting", ERR_REPORT_ONCE);
@@ -436,11 +437,13 @@ void sf_readsettings(void)
 	AcHeight = sf_GetProfileInt("Window", "AcHeight", AcHeight);
 
 	m_frequency = sf_GetProfileInt("Audio", "Frequency", m_frequency);
-	m_v6scale = sf_GetProfileInt("Display", "Infocom V6 Scaling", 2);
+	m_v6scale_x = sf_GetProfileInt("Display", "Infocom V6 Scaling X", 2);
+	m_v6scale_y = sf_GetProfileInt("Display", "Infocom V6 Scaling Y", 2);
 	m_gfxScale_w = 1.0;
 	m_gfxScale_h = 1.0;
-	m_defaultFore = (sf_GetProfileInt("Display", "Foreground", 0xffffff));
-	m_defaultBack = (sf_GetProfileInt("Display", "Background", 0x800000));
+	sf_initcolours();
+	m_defaultFore = (sf_GetProfileInt("Display", "Foreground", m_colours[7]));
+	m_defaultBack = (sf_GetProfileInt("Display", "Background", m_colours[10]));
 	m_morePrompts =
 	    sf_GetProfileInt("Display", "Show More Prompts", 1) ? true : false;
 	m_gamma = sf_GetProfileDouble("Display", "Gamma", DEFAULT_GAMMA);
@@ -644,14 +647,25 @@ void os_init_screen(void)
 
 	/* Set the graphics scaling */
 	if (sf_IsInfocomV6() || (story_id == BEYOND_ZORK)) {
-		m_gfxScale_w = (double)m_v6scale;
-		m_gfxScale_h = (double)m_v6scale;
+		/* EGA's 640x200 mode has rectangular pixels that are
+		 * narrower than they are tall.  For Infocom's V6 EGA
+		 * graphics to look correct, the X scale must be halved.
+		 */
+		if (reso != NULL && (reso->px == 640 && reso->py == 200))
+			m_v6scale_x /= 2.0;
+
+		m_gfxScale_w = (double)m_v6scale_x;
+		m_gfxScale_h = (double)m_v6scale_y;
 	} else {
 		m_gfxScale_w = 1.0;
 		m_gfxScale_h = 1.0;
 	}
-	m_gfxScale_w *= (double) AcWidth /640.0;
-	m_gfxScale_h *= (double) AcHeight /400.0;
+
+	if (m_xscale > 0.0)
+		m_gfxScale_w *= m_xscale;
+
+	if (m_yscale > 0.0)
+		m_gfxScale_h *= m_yscale;
 
 	/* Set the configuration */
 	if (z_header.version == V3) {
@@ -714,6 +728,12 @@ void os_init_screen(void)
 			z_header.flags &= ~SOUND_FLAG;
 	}
 
+	if (option_disable_color) {
+		z_header.default_foreground = WHITE_COLOUR;
+		z_header.default_background = BLACK_COLOUR;
+		if (z_header.flags & COLOUR_FLAG) z_header.flags &= ~COLOUR_FLAG;
+	}
+
 	if (z_header.version >= V5) {
 		zword mask = 0;
 		if (z_header.version == V6)
@@ -728,6 +748,15 @@ void os_init_screen(void)
 }
 
 
+static void print_c_string (const char *s)
+{
+	zchar c;
+
+	while ((c = *s++) != 0)
+		os_display_char (c);
+} /* print_c_string */
+
+
 /*
  * os_fatal
  *
@@ -746,16 +775,12 @@ void os_fatal(const char *s, ...)
 
 	if (sdl_active) {
 		os_set_text_style(0);
-		os_display_string((zchar *) "\n\n");
+		print_c_string("\n\n");
 		os_beep(BEEP_HIGH);
 		os_set_text_style(BOLDFACE_STYLE);
-
-		os_display_string((zchar *) "Fatal error: ");
+		print_c_string("Fatal error: ");
 		os_set_text_style(0);
-		os_display_string((zchar *) s);
-		os_display_string((zchar *) "\n\n");
-		new_line();
-		flush_buffer();
+		print_c_string(s);
 	}
 
 	if (f_setup.ignore_errors) {
@@ -773,7 +798,7 @@ void os_fatal(const char *s, ...)
 		SDL_Quit();
 	}
 	sf_cleanup_all();
-	exit(EXIT_FAILURE);
+	os_quit(EXIT_FAILURE);
 }
 
 
@@ -1101,6 +1126,7 @@ static FILE *findfromlist(int ispic, int num, int *size)
 
 void os_init_setup(void)
 {
+	f_setup.config_file = strdup(CONFIG_FILE);
 	sf_setdialog();
 	sf_initloader();
 	sdl_active = FALSE;
