@@ -37,6 +37,7 @@
 
 #ifdef UNIX
 #include <unistd.h>
+#include <getopt.h>
 #endif
 
 extern f_setup_t f_setup;
@@ -134,7 +135,6 @@ int m_random_seed = -1;
 int m_fullscreen = -1;
 int m_reqW = 0, m_reqH = 0;
 int m_vga_fonts = 0;
-extern char *m_setupfile;
 extern char m_names_format;
 static char user_names_format = 0;
 extern char *m_reslist_file;
@@ -151,6 +151,7 @@ static char *info[] = {
 	"-A   watch attribute testing",
 	"-b <colourname> background colour",
 	"-c # context lines",
+	"-C <file> load this configuration file",
 	"-d   disable color",
 	"-f <colorname> foreground colour",
 	"-F   fullscreen mode",
@@ -196,7 +197,7 @@ static char *footer =
     "More options and information are in the manual page.  Type \"man sfrotz\".\n";
 
 
-#define WIDCOL 40
+#define WIDCOL 42
 static void usage(int type)
 {
 	char **p;
@@ -248,14 +249,23 @@ static void usage(int type)
 
 static const char *progname = NULL;
 
-/*
 extern char *optarg;
 extern int optind;
-*/
 
 extern int m_timerinterval;
 
-static char *options = "@:%aAb:B:c:df:FH:iI:l:L:m:N:oOPqr:s:S:tTu:vVW:xXZ:";
+double m_xscale = 0;
+double m_yscale = 0;
+
+static int xscale_flag = 0;
+static int yscale_flag = 0;
+
+static char *options = "@:%aAb:B:c:C:df:FH:iI:l:L:m:N:oOPqr:s:S:tTu:vVW:xXZ:";
+static struct option long_options[] = {
+	{"xscale", required_argument, &xscale_flag, 1},
+	{"yscale", required_argument, &yscale_flag, 1},
+	{0, 0, 0, 0}
+};
 
 static int limit(int v, int m, int M)
 {
@@ -273,14 +283,21 @@ static void parse_options(int argc, char **argv)
 
 	do {
 		int num = 0, copt = 0;;
+		int option_index = 0;
 
-		c = zgetopt(argc, argv, options);
+		c = getopt_long(argc, argv, options, long_options, &option_index);
 
-		if (zoptarg != NULL) {
-			num = atoi(zoptarg);
-			copt = zoptarg[0];
+		if (optarg != NULL) {
+			num = atoi(optarg);
+			copt = optarg[0];
 		}
 
+		if (c == 0) {
+			if (xscale_flag)
+				m_xscale = atof(optarg);
+			if (yscale_flag)
+				m_yscale = atof(optarg);
+		}
 		if (c == '%')
 			m_localfiles = true;
 		if (c == 'a')
@@ -288,11 +305,16 @@ static void parse_options(int argc, char **argv)
 		if (c == 'A')
 			f_setup.attribute_testing = 1;
 		if (c == 'b')
-			user_background = getcolor(zoptarg);
+			user_background = getcolor(optarg);
 		if (c == 'B')
 			option_scrollback_buffer = num;
 		if (c == 'c')
 			f_setup.context_lines = num;
+		if (c == 'C') {
+			/* originally allocated in os_init_setup() */
+			free(f_setup.config_file);
+			f_setup.config_file = strdup(optarg);
+		}
 		if (c == 'd')
 			option_disable_color = 1;
 		if (c == 'm')
@@ -300,9 +322,9 @@ static void parse_options(int argc, char **argv)
 		if (c == 'N')
 			user_names_format = copt;
 		if (c == '@')
-			m_reslist_file = zoptarg;
+			m_reslist_file = optarg;
 		if (c == 'f')
-			user_foreground = getcolor(zoptarg);
+			user_foreground = getcolor(optarg);
 		if (c == 'F')
 			m_fullscreen = 1;
 		if (c == 'H')
@@ -317,7 +339,7 @@ static void parse_options(int argc, char **argv)
 			f_setup.left_margin = num;
 		if (c == 'L') {
 			f_setup.restore_mode = TRUE;
-			f_setup.tmp_save_name = strdup(zoptarg);
+			f_setup.tmp_save_name = strdup(optarg);
 		}
 		if (c == 'q')
 			m_no_sound = 1;
@@ -415,20 +437,27 @@ void os_process_arguments(int argc, char *argv[])
 {
 	char *p;
 
-	zoptarg = NULL;
+	optarg = NULL;
 
 	sf_installhandlers();
+
+	/* Parse command line first because we might be told to load a
+	 * different config file.  Then read from that config file.
+	 * Finally we parse the command line again because options
+	 * specified there must override the config file.
+	 */
+	parse_options(argc, argv);
 	sf_readsettings();
 	parse_options(argc, argv);
 
-	if (argv[zoptind] == NULL) {
+	if (argv[optind] == NULL) {
 		usage(USAGE_NORMAL);
 		os_quit(EXIT_SUCCESS);
 	}
-	f_setup.story_file = strdup(argv[zoptind]);
+	f_setup.story_file = strdup(argv[optind]);
 
-	if (argv[zoptind + 1] != NULL)
-		f_setup.blorb_file = argv[zoptind + 1];
+	if (argv[optind + 1] != NULL)
+		f_setup.blorb_file = argv[optind + 1];
 
 	/* Strip path and extension off the story file name */
 	f_setup.story_name = new_basename(f_setup.story_file);
@@ -451,7 +480,7 @@ void os_process_arguments(int argc, char *argv[])
 			*p = '\0';	/* extension removed */
 		}
 	}
-	f_setup.story_path = new_dirname(argv[zoptind]);
+	f_setup.story_path = new_dirname(argv[optind]);
 
 	/* Create nice default file names */
 	f_setup.script_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SCRIPT) + 1) * sizeof(char));
